public class Game
{	private Board b;
	private Player players[];
	private int currentPlayer,gameNumber;
	private String[] pieces;
	private GameLog log;
	private Server server;
	static final long serialVersionUID=(long)0.0;
	public Game(Server s,Board brd,Player[] p,int n,String[] pi)
	{  	b=brd;gameNumber=n;pieces=pi;
      	players=new Player[brd.players];
      	for(int i=0;i<p.length;i++)
      		players[i]=p[i];
      	currentPlayer=1;
      	server=s;
      	log=new GameLog(b);
	}
	public synchronized boolean validMove(int player,
			int i,int j,int x,int y,String dropType)
	{	while(player!=currentPlayer)
		{	try
			{	wait();
			}catch(InterruptedException e)
			{	e.printStackTrace();
			}
		}
		boolean valid=false,isADrop=i==99&&j==99;
		if(isADrop)
		{	valid=b.canDrop(dropType,x,y);
		}
		else valid=b.canMove(i,j,x,y);
		if(valid)
		{	if(isADrop)b.drop(dropType,x,y);
			else b.move(i,j,x,y);
			log.addMove(b);
			currentPlayer=b.turn;
			players[currentPlayer-1].otherPlayerMoved(i,j,x,y,dropType);
		    notify();//tell waiting player to continue
			if(gameOver())endGame();
			return true;
		}
		return false;
	}
	public synchronized void resign(int j)
	{	b.resign(j);
		for(int i=0;i<players.length;i++)
			players[i].resign(j);
		log.addMove(b);
		endGame();
	}
	public synchronized void chat(String s)
	{	for(int i=0;i<players.length;i++)
		{	players[i].sendString("Game Chat");
			players[i].sendString(s);
		}
	}
	public boolean gameOver()
	{	return b.gameOver();
	}
	public int getNumber()
	{	return gameNumber;
	}
	public String[] getPieces()
	{	return pieces;
	}
	public Board board()
	{	return b;
	}
	public synchronized void abortGame()
	{	for(int i=0;i<players.length;i++)
			players[i].abortGame();
		server.removeGame(this);
	}
	public synchronized void endGame()
	{	for(int i=0;i<players.length;i++)
			players[i].endGame();
		server.removeGame(this);
	}
	public boolean includesPlayer(Player p)
	{	for(int i=0;i<players.length;i++)
			if(players[i].equals(p))
				return true;
		return false;
	}
	public String playerStrings()
	{	String r="";
		for(int i=0;i<players.length;i++)
			r+=players[i].getPlayerName()+"\n";
		return r;
	}
	public synchronized void takeBackRequest(Player p)
	{	for(int i=0;i<players.length;i++)
		{	Player d=players[i];
			if(!d.equals(p))d.sendString("Takeback Request");
		}
	}
	public synchronized void drawOffer(Player p)
	{	for(int i=0;i<players.length;i++)
		{	Player d=players[i];
				if(!d.equals(p))d.sendString("Draw Offer");
		}
	}
	public synchronized void takeBackAccepted(Player p)
	{	int ammt=1;
		String cur=players[currentPlayer-1].getPlayerName();
		if(!cur.equals(p.getPlayerName()))
		{	ammt=2;
			b=log.takeBackMove().returnClone();
		}
		b=log.takeBackMove().returnClone();
		currentPlayer=b.turn;
		for(int i=0;i<players.length;i++)
			players[i].takeBackMove(ammt);
	}
	public synchronized void drawAccepted()
	{	b.draw();log.addMove(b);
		for(int i=0;i<players.length;i++)
			players[i].sendString("Game Drawn");
		endGame();
	}
}