import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class GameWindow extends JFrame
{	JTextField msg;
	BoardPanel boardPanel;
	Board board;
	GameLog log;
	int player;
	Client client;
	JTextArea display;
	JTextField textEntry;
	PlayerList players;
	JButton resignButton,undoButton,offerDrawButton;
	static final long serialVersionUID=(long)0.0;
	public GameWindow(Board b,Client c,int p,String pl)
	{	super("Game- FIGS");
		players=new PlayerList(pl,8);
		client=c;
		board=b.returnClone();
		player=p;
		log=new GameLog(b);
		boardPanel=new BoardPanel(600,client,this);
	    msg=new JTextField();
	    msg.setText(board.message());
	    msg.setEditable(false);
	    display=new JTextArea(10,20);
		display.setLineWrap(true);
		display.setEditable(false);
		display.setForeground(Color.blue);
		JScrollPane displayPane=new JScrollPane(display,
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		textEntry=new JTextField();
		textEntry.setEnabled(true);
		textEntry.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	String s=textEntry.getText();
				if(!s.equals(""))
				{	textEntry.setText("");
					client.gameChat(s);
				}
			}
		});
		undoButton=new JButton("Request Undo");
		undoButton.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	if(!board.gameOver())
					client.takeBackMoveRequest();
			}
		});
		resignButton=new JButton("Resign");
		resignButton.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	if(!board.gameOver())
					client.resign();
			}
		});
		offerDrawButton=new JButton("Offer a Draw");
		offerDrawButton.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	if(!board.gameOver())
					client.offerDraw();
			}
		});
		JPanel text=new JPanel(),textAndPlayers=new JPanel(),
		boardAndMessage=new JPanel(),buttons=new JPanel();
		text.setLayout(new BorderLayout());
		text.add(displayPane);
		text.add(textEntry,"South");
		buttons.setLayout(new GridLayout(0,1));
		buttons.add(undoButton);
		buttons.add(offerDrawButton);
		buttons.add(resignButton);
		textAndPlayers.setLayout(new BorderLayout());
		textAndPlayers.add(text);
		textAndPlayers.add(players,"West");
		textAndPlayers.add(buttons,"East");
		boardAndMessage.setLayout(new BorderLayout());
		boardAndMessage.add(msg,"North");
		boardAndMessage.add(boardPanel);
	    Container cc=getContentPane();
		addMenu();
	    cc.setLayout(new BorderLayout());
	    cc.add(boardAndMessage);
	    cc.add(textAndPlayers,"South");
	    pack();
	    setVisible(true);
	    setResizable(false);
	}
	public void makeMove(int i,int j,int x,int y)
	{	board.move(i,j,x,y);
		log.addMove(board);
		boardPanel.repaint();
	}
	public void makeDrop(int i,int j,String dropType)
	{	board.drop(dropType,i,j);
		log.addMove(board);
		boardPanel.repaint();
	}
	public void addMenu()
	{	JMenuItem quit,takeBackMove,resign,
		gameOptions,visualOptions,flipBoard,gameHelp,about;
		quit=new JMenuItem("Quit");
		takeBackMove=new JMenuItem("Request Undo");
		resign=new JMenuItem("Resign");
		gameOptions=new JMenuItem("Gameplay Options");
		visualOptions=new JMenuItem("Visual Options");
		flipBoard=new JMenuItem("Flip Board");
		gameHelp=new JMenuItem("Contents");
		about=new JMenuItem("About");
		quit.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	client.quit();
			}
		});
		takeBackMove.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	client.takeBackMoveRequest();
			}
		});
		resign.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	client.resign();
			}
		});
		gameOptions.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	//new gameOptionsMenu();
			}
		});
		visualOptions.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	client.visualOptions.packStuff();
			}
		});
		flipBoard.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	boardPanel.flipBoard();
			}
		});
		gameHelp.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	new HelpMenu();
			}
		});
		about.addActionListener(new ActionListener()
		{	public void actionPerformed(ActionEvent e)
			{	new AboutPopup();
			}
		});
		JMenuBar mb=new JMenuBar();
		JMenu file,actions,options,help;
		file=new JMenu("File");
	    actions=new JMenu("Game Actions");
	    options=new JMenu("Options");
	    help=new JMenu("Help");
	    file.add(quit);
	    actions.add(takeBackMove);
	    actions.add(resign);
	    options.add(gameOptions);
	    options.add(visualOptions);
	    options.add(flipBoard);
	    help.add(gameHelp);
	    help.add(about);
	    mb.add(file);
	    mb.add(actions);
	    mb.add(options);
	    mb.add(help);
	    setJMenuBar(mb);
	}
	public void takeBackMove(int i)
	{	for(int j=0;j<i;j++)
		{	board=log.takeBackMove().returnClone();
			boardPanel.selected=false;
			boardPanel.repaint();
		}
	}
	public void playerResigned(int i)
	{	board.resign(i);
		log.addMove(board);
		boardPanel.selected=false;
		boardPanel.repaint();
	}
	public void draw()
	{	board.draw();
		log.addMove(board);
		boardPanel.selected=false;
		boardPanel.repaint();
	}
	public void resetTheSize()
	{	setSize(boardPanel.width+10,boardPanel.height+240);
	}
	public boolean myTurn()
	{	return board.turn==player;
	}
	public void display(String s)
	{	display.append(s+"\n");
	}
}